@@ -16,14 +16,6 @@
 #include "../optgroup.h"
 #include "../lib/memalign.h"
 
-#ifndef IOCB_FLAG_HIPRI
-#define IOCB_FLAG_HIPRI	(1 << 2)
-#endif
-
-#ifndef IOCTX_FLAG_IOPOLL
-#define IOCTX_FLAG_IOPOLL	(1 << 0)
-#endif
-
 static int fio_libaio_commit(struct thread_data *td);
 
 struct libaio_data {
@@ -52,7 +44,6 @@ struct libaio_data {
 struct libaio_options {
 	void *pad;
 	unsigned int userspace_reap;
-	unsigned int hipri;
 };
 
 static struct fio_option options[] = {
@@ -65,15 +56,6 @@ static struct fio_option options[] = {
 		.category = FIO_OPT_C_ENGINE,
 		.group	= FIO_OPT_G_LIBAIO,
 	},
-	{
-		.name	= "hipri",
-		.lname	= "High Priority",
-		.type	= FIO_OPT_STR_SET,
-		.off1	= offsetof(struct libaio_options, hipri),
-		.help	= "Use polled IO completions",
-		.category = FIO_OPT_C_ENGINE,
-		.group	= FIO_OPT_G_LIBAIO,
-	},
 	{
 		.name	= NULL,
 	},
@@ -91,19 +73,12 @@ static inline void ring_inc(struct libaio_data *ld, unsigned int *val,
 static int fio_libaio_prep(struct thread_data fio_unused *td, struct io_u *io_u)
 {
 	struct fio_file *f = io_u->file;
-	struct libaio_options *o = td->eo;
-	struct iocb *iocb;
-
-	iocb = &io_u->iocb;
+	struct iocb *iocb = &io_u->iocb;
 
 	if (io_u->ddir == DDIR_READ) {
 		io_prep_pread(iocb, f->fd, io_u->xfer_buf, io_u->xfer_buflen, io_u->offset);
-		if (o->hipri)
-			iocb->u.c.flags |= IOCB_FLAG_HIPRI;
 	} else if (io_u->ddir == DDIR_WRITE) {
 		io_prep_pwrite(iocb, f->fd, io_u->xfer_buf, io_u->xfer_buflen, io_u->offset);
-		if (o->hipri)
-			iocb->u.c.flags |= IOCB_FLAG_HIPRI;
 	} else if (ddir_sync(io_u->ddir))
 		io_prep_fsync(iocb, f->fd);
 
@@ -366,42 +341,12 @@ static void fio_libaio_cleanup(struct thread_data *td)
 	}
 }
 
-static int fio_libaio_old_queue_init(struct libaio_data *ld, unsigned int depth,
-				     bool hipri)
-{
-	if (hipri) {
-		log_err("fio: polled aio not available on your platform\n");
-		return 1;
-	}
-
-	return io_queue_init(depth, &ld->aio_ctx);
-}
-
-static int fio_libaio_queue_init(struct libaio_data *ld, unsigned int depth,
-				 bool hipri)
-{
-#ifdef __NR_sys_io_setup2
-	int ret, flags = 0;
-
-	if (hipri)
-		flags |= IOCTX_FLAG_IOPOLL;
-
-	ret = syscall(__NR_sys_io_setup2, depth, flags, NULL, NULL,
-			&ld->aio_ctx);
-	if (!ret)
-		return 0;
-	/* fall through to old syscall */
-#endif
-	return fio_libaio_old_queue_init(ld, depth, hipri);
-}
-
 static int fio_libaio_post_init(struct thread_data *td)
 {
 	struct libaio_data *ld = td->io_ops_data;
-	struct libaio_options *o = td->eo;
-	int err = 0;
+	int err;
 
-	err = fio_libaio_queue_init(ld, td->o.iodepth, o->hipri);
+	err = io_queue_init(td->o.iodepth, &ld->aio_ctx);
 	if (err) {
 		td_verror(td, -err, "io_queue_init");
 		return 1;
